%DiscreteB - Bayesian version of 
%Discrete Probability distribution class, representing
%random binary 1-of-M vector Z,
%with length M, and with exactly one element =1.
%
%Eqivalently, this RV can also be coded as the index
%of the binary element that is =1, i.e. as iZ \in {1...M}
%
%Several DiscreteB objects may be collected in a multidimensional array,
%
%The probability of a binary 1 at position k is
%P[Z(k)==1] = Q(k).
%
%In the Bayesian approach, 
%the probability vector parameter Q is regarded as a Random Vector,
%with a Dirichlet distribution,
%defined by hyperparameter vector Alfa,
%which is adapted to training data.
%
%Object Properties:
%Weight=    Dirichlet hyperparameter vector Alfa, such that
%   Q has a Dirichlet density function, i.e.
%   p(Q | Alfa)= Dir(Q | Alfa)= c* prod_k Q(k).^(Alfa(k)-1);
%
%   Weight is stored internally as COLUMN vector,
%   because resulting binary random vectors are columns.
%
%Arne Leijon 2009-05-06 ****** NOT fully implemented ******

classdef DiscreteB
    properties
        Weight=1;%Dirichlet hyperparameter Alfa
%           Initial default for a non-informative prior: Weight==1, to make
%           p(Q(k)) = 1, constant.
    end
    properties (Dependent=true)
        DataSize;%number of possible discrete random values
        Mean;%posterior mean = E( Z ), where Z is binary selector vector
    end
    methods
        function pD=DiscreteB(pMass)%constructor
%           Input:
%           pMass may be a DiscreteB object, then it is simply copied, otherwise
%           pMass= vector or matrix with prior Dirichlet Alfa values.
%               if pMass is a row or column vector, one DiscreteB object is created
%               if pMass is a matrix,
%           each COLUMN defines a Weight vector,
%           and one DiscreteB object is created for each column.
%           If nargin==0, a standard simple DiscreteB object is created.
%
%           Output:
%           pD=    a DiscreteB object or row vector of such objects
%          Arne Leijon 2007-05-22 tested

            if nargin >= 1
                if isa(pMass,'DiscreteB')
                    pD=pMass;
                else
                    if size(pMass,1) == 1 %row vector with row-wize prob. mass values
                        pMass=pMass';%transpose to use columnwise
                    end;
                    pD=repmat(pD,1,size(pMass,2));%row vector of objects
                    for i=1:numel(pD)
                        pD(i).Weight=pMass(:,i);%build struct
                    end;
                end;
            end;
        end;
        pMass=double(pD);%convert Weight back to a real matrix; inverse constructor
        iZ=rand(pD,nData);%generate nData random samples in integer index form
%        Z=randbin(pD,nData);%generate nData random binary 1-of-M vectors
        p=prob(pD,Z);%give probability mass of observed samples
        mlQ=meanlogprob(pD);%=E[ln Q] ******** to be implemented
        H=entropy(pD);%entropy H(Z) of discrete random variable Z
%*****  Training Method:
        [pD,LL,pDWinit]=adapt(pDprior,pZ);%adjust the object to given training data.
%           Training data may be either 1-of-K binary (column) vectors,
%           or probability (column) vectors, calculated externally.
    end
    methods%for dependent properties, called only for one object at a time
        function s=get.DataSize(pD)
            s=length(pD.Weight);
        end
        function mu=get.Mean(pD)
            mu=pD.Weight./sum(pD.Weight);
        end;
        function mu=mean(pD)%for backward compatibility
            mu=pD.Mean;
        end
    end
    methods (Access='private')
        iz=bin2int(bz);%convert binary sample vectors to integer index form
        bz=int2bin(iz);%convert integer random samples to binary vector form
    end;
end
       