%p=prob(pD,Z)
%method to give the probability of a data sequence,
%assumed to be drawn from given Discrete Distribution(s).
%
%Input:
%pD=    DiscreteB object or array of DiscreteB objects
%Z=     a sequence of observed Discrete random data,
%   EITHER
%       a row vector with integer-coded data
%       (may be real-valued, but is always rounded to integer values)
%   OR
%       a matrix with sequence of binary 1-of-M (column) vectors, with
%       exactly one element == 1.
%       (may be real-valued, but is rounded to binary values)
%
%Result:
%p=     array with probability values for each element in Z,
%       for each given DiscreteB object.
%       size(p)== [length(pD),size(x,2)], if pD is one-dimensional vector
%       size(p)== [size(pD),size(x,2)], if pD is multidim array
%
%Arne Leijon 2009-05-06 tested

function p=prob(pD,Z)
if size(Z,1)>1
    Z=bin2int(Z>0.5);%make sure it is binary, and convert to integer
else
    Z=round(Z);%make sure it is integer
end;

pDsize=size(pD);%size of DiscreteB array
pDlength=prod(pDsize);%number of DiscreteB objects
nZ=size(Z,2);%number of observed data
p=zeros(pDlength,nZ);%zero prob if Z is out of range
for i=1:pDlength%for all objects in pD
    iDataOK=(Z>=1) & (Z <=length(pD(i).Weight));%within range of the DiscreteB
    mu=pD(i).Mean;%P(z) normalized
    p(i,iDataOK )=mu(Z(iDataOK))';
end;
if pDlength>1
    p=squeeze(reshape(p,[pDsize,nZ]));%restore array format
end;
