%r=initMixProb(pD,x);
%determine crude initial assignment of training data
%to the components of a GMMB object,
%needed as starting point for training.
%
%Input:
%pD=    a single GMMB object
%x=     matrix with observed vectors stored columnwise
%
%Result:
%r=     normalized probability values
%r(i,n)=    P[Z(n)==i |x(:,n)], where
%       Z(n) is index of Gaussian component, that
%       may have generated x(:,n).
%       r(i,n) may be binary 0/1, indicating a hard assignment
%
%Requires: VQ class
%
%Arne Leijon 2007-05-25 slightly tested
%Gustav Henter 2007-05-30
%
% Note: The centers etc. of the clusters are not given by this function,
% only the partitioning.

function r = initMixProb(pD,x)

if numel(pD) > 1
    error('Works only for single object');
end

N = size(x,2);
K = length(pD.Gaussians);

% Initiate r-values using the generalized Lloyd/K-means algorithm
[vq,varC] = create(VQ,x,K);

r = zeros(K,N);%bug corrected Zhanyu Ma, 2008-03-06

% Hard assignment (nearest neighbor)
ri = encode(vq,x);
for i=1:K%K is typically <<N, Arne Leijon, 2008-03-07
r(i,ri==i)=1;
end;
% for n = 1:N
%     r(ri(n),n) = 1;
% end