%[f,logS]=prob(pD,x) gives posterior probability densities for given vectors
%to be drawn from a given GMMB object or array of such objects.
%
%Input:
%pD=    GMMB object or multidim array of such objects
%x=     matrix with given vectors, stored columnwise
%
%Result:
%f=     proportional to probability densities for x
%       size(f)== [numel(pD),size(x,2)]
%       max f value always =1
%logS=  scalar log scalefactor to make sure f is within numerical limits
%       f*exp(logS)=   true probability densities for x
%
%This representation is necessary because the probability density may be
%extremely small for random vectors with many elements
%  
%Arne Leijon 2009-05-06 NOT tested

function [f,logS]=prob(pD,x)
nObj=numel(pD);%number of GMMB array elements
nx=size(x,2);%number of observed vectors
f=zeros(nObj,nx);%init result storage
logScale=zeros(nObj,1);%log scale factors
for k=1:nObj%each GMMB object
    [subProb,logScale(k)]=prob(pD(k).Gaussians,x);%prob.s from all sub-Gaussians
    %********output subProb also, to save time later???
    w=mean(pD(k).MixWeight);%posterior mean weights
    f(k,:)=w'*subProb;%weighted sum
end;
logS=max(logScale);
fscale=max(max(f));
scale=exp(logScale-logS)./fscale;%same scaling for all objects
logS=logS+log(fscale);
if logS>-Inf
    f=f.*repmat(scale,1,nx);%f*exp(logS)= true prob.density
end;
