%GaussB - Probability distribution class, representing
%Gaussian random vector, allowing for Bayesian training.
%
%The Bayesian training approach means that the parameters
%defining the object are defined by Probability Distributions,
%NOT by point estimates as in the standard Maximum Likelihood approach.
%
%The multidimensional Gaussian density is defined by 
%a mean vector (mu) and a precision matrix (Lambda) (inverse covariance):
%
%p(x)= gaussdens(x | mu, Lambda^(-1) ), where
%(mu, Lambda) are regarded as random variables,  
%specified by the conjugate Gauss-Wishart distribution, as
%
%p(mu, Lambda | m, beta, W, nu)= gaussdens(mu | m, (beta Lambda)^(-1)) *
%           Wishart(Lambda | W, nu)
%defined by "hyperparameters" m, beta, W, nu, stored as object properties.
%
%***Usage:
%gD = GaussB; 
%   creates a single default distribution.
%gD = GaussB(pD); 
%   just copies the given object.
%
%Arne Leijon 2009-05-06 NOT fully implemented

classdef GaussB
    properties (Dependent=true)
        DataSize;%integer length of random vector
    end
    properties%=hyperparameters of the Gauss-Wishart parameter distributions:
        Mean=0;%mean of random mean parameter vector, also =mean of the GaussB
        MeanPrec=0;%scale factor for precision of random mean vector (Bishop beta)
        WishartW=1;%Wishart W matrix for random precision matrix (Bishop W)
        WishartDF=0;%Wishart degrees of freedom (Bishop nu)
        
        UserData=[];%anything
    end
    methods%public
        function gD = GaussB(pD)%constructor
            if (nargin > 0)
                if isa(pD,'GaussB')
                    gD= pD;%copy it without transformation
                else
                    error('Argument must be a GaussB object');
                end;
            end;
        end
        function s=get.DataSize(pD)%for dependent property
            s=length(pD.Mean);
        end
        %x=rand(pD,nData);%generate random vector samples
        [f,logS]=prob(pD,x);%probability densities for observed vector samples
        logP=logprob(pD,x);%log probability densitites for observed vector samples
        plotH=plotCross(pD,proj,colors);%for user entertainment
        %plotProb;%  3D plot of probability density function???
%**** Training Methods ****
        pD=init(pD,x);%initialize crudely to conform with given data
        [pD,LL]=adapt(pDprior,x,pZ,lPz,pDWeight,pDWinit,E2k);%adjust hyperparameters to training data
        [logP E2k]=meanlogprob(pD,x);%E_{mu,Lambda}[log(probability densities for x)]
    end
    methods (Access='private')
        OK=propertiesOK(gD);%validate given combination of properties before calc
    end        
end
