%[pD,LL,pDprior]=adapt(pDprior,x,pZ)
%method to adapt a GaussB object or array of such objects
%using training data.
%
%Input:
%pDprior=   GaussB object or array of GaussD objects
%           with prior property values
%x=         training data vectors, stored columnwise
%pZ=        component responsibilities
%pZ(i,n)=   P[ Z(n)==i | x(:,n) ], 
%           where Z(n) is index of the GaussB element
%           that might have generated x(:,n)
%
%Result:
%pD=        adapted version of the GaussB object(s)
%LL=        low bound on Log-Likelihood of training data
%           in the variational approximation.
%           (Bishop, 10.2.2)
%
%Method: Bishop(2006), sec. 10.2.
%
%Arne Leijon 2007-05-25 slightly tested
%***** Svante Stadler

function [pD,LL]=adapt2(pDprior,x,pZ,lPz)

% pD1=pDprior.MixWeight;
% a1=double(pD1);%collect all Weight column vectors in a matrix
% [K1,N1] = size(a1);
% 
% mlQ1 = psi(a1) - repmat(psi(sum(a1)),K1,1);
% 
% pD

minWeight=0.001;%limit to avoid underflow...???
pD=pDprior;
nObj=numel(pD);
[nD,nX]=size(x);
if nargin<3%should happen only when nObj==1
    if nObj==1
        pZ=ones(nObj,nX);%equal weight for all objects
    else
        error('GaussB array needs external component weights');
    end;
end;
LL=zeros(nObj,1);%space
for k=1:nObj%for all GaussB objects
    Nk=sum(pZ(k,:),2);%Eq. 10.51:effective number of obs. for this GaussB object
    if Nk>minWeight%we still have some weight on this component
        %UPDATE pD
        b0 = pD(k).MeanPrec; v0 = pD(k).WishartDF;
%         m0 = 0;
        m0=pD(k).Mean;
        W0i = inv(pD(k).WishartW);
        xk = (1/Nk) * sum(x.*repmat(pZ(k,:),nD,1),2);
        x0 = x - repmat(xk,1,nX);
        bk = b0 + Nk;
        mk = (b0*m0 + Nk*xk)/bk;
        NkSk = (x0.*repmat(pZ(k,:),nD,1))*x0';
        Wvar = b0*Nk/bk * (xk-m0)*(xk-m0)';
        Wi = W0i + NkSk + Wvar;
        
        %set data
        pD(k).Mean      = mk;
        pD(k).WishartW  = inv(Wi);
        pD(k).MeanPrec  = b0 + Nk;
        pD(k).WishartDF = v0 + Nk;
        
    else%**** the new values may cause numerical underflow
        pD(k)=pDprior(k);%just copy prior
        pD(k).MeanPrec=pDprior(k).MeanPrec+minWeight;%to avoid underflow
        pD(k).WishartDF=pDprior(k).WishartDF+minWeight;%to avoid underflow
    end;
    %%%%%%%%%%%%
    if nargout>1%calculate log-likelihood bound
        scale = max(max( pD(k).WishartW));
        Ws =  pD(k).WishartW * (1/scale);
        K = size(pD(k).WishartW,1)*log(scale);
        E2 = nD*log(2) + log(det(Ws))+ K + sum(psi((pD(k).WishartDF +1-(1:nD))/2));
        E3 = -pD(k).WishartDF*trace((NkSk/Nk)*pD(k).WishartW);
        E4=-nD/bk;
        x_m = x-repmat(pD(k).Mean,1,nX);
        E5=-pD(k).WishartDF*sum((x_m).*(pD(k).WishartW*(x_m)));
        E6=-nD*log(2*pi);
        L1k(:,:,k)=Nk*(repmat(E2,1,nX)+ repmat(E4,1,nX)+ repmat(E3,1,nX)+E5+ repmat(E6,1,nX));
        %%%%%%%%%%%%%%%%
        %LL(k)= %*******NOT yet implemented
    end;
    %%%%%%%%%%%%
end;
% L1=0.5*sum(L1k)
LL=sum(LL);%across all Gaussian components