%pD=init(pD,x);
%initialize a GaussB object or array of such objects
%to conform with a set of given training vectors.
%
%NOTE: NO training is performed. 
%Properties are only set to reasonable default prior values.
%
%Input:
%pD=    single GaussB object or multidim array of GaussD objects
%x=     matrix with observed vectors stored columnwise
%
%Result:
%pD=    initialized GaussB object (array)
%       size(pD)== same as input
%
%Method:
%       Mean vectors set at data mean point,
%       WishartW set at very large value.
%       MeanPrec and WishartDF set to least possible values,
%       indicating that no adaptation has actually been done.
%  
%Arne Leijon 2009-05-06 slightly tested

function pD=init(pD,x)
priorBeta=0.001;%MeanPrec, least possible...???
priorDF=0.001;%WishartDF =priorDF+D-1; least possible...???
%***** We want the prior to control training as little as possible, BUT
%***** set slightly greater than 0 to avoid numerical underflow later???

D=size(x,1);%data dimensionality
priorW=(realmax/10).^(1./D);%inv(WishartW) near zero, det(W)<Inf
%to avoid numerical 0*Inf later...

mx=mean(x,2);%total mean of all training data
if size(x,2)>1
    W=diag(1./var(x,0,2));%total precision
else
    W=eye(D)*priorW;%default in case too few data??????
end;
nObj=numel(pD);
for i=1:nObj
%    pD=set(pD,'Mean',mx,'MeanPrec',priorBeta,'WishartW',W,'WishartDF',priorDF+D-1);
    pD(i).Mean=mx;
    pD(i).MeanPrec=priorBeta;
    pD(i).WishartW=W;
    pD(i).WishartDF=priorDF+D-1;
end;