%logP=meanlogprob(pD,x) gives
%mean of log(probability densities) for given vectors,
%assumed to be drawn from a given GaussB object.
%
%Input:
%pD=    GaussB object or array of such objects
%x=     matrix with given vectors, stored columnwise
%
%Result:
%logP=  E[log(probability densities for x)],
%       marginalized over random mean and precision
%       size(logP)== [numel(pD),size(x,2)].
%
%Method:
%Ref: Bishop(2006), 10.2
%  
%Arne Leijon 2007-05-25 slightly tested
%Ermin Kozica 2007-06-04
%Svante Stadler 2007-06-07 - bugfix, optimization

function [logP E2k]=meanlogprob(pD,x)
nObj=numel(pD);%number of GaussB objects in array
nx=size(x,2);%number of observed vectors
logP=zeros(nObj,nx);%space for result
for i=1:nObj%for all GaussB objects in pD
    D=length(pD(i).Mean);%GaussB random vector length
    if D==size(x,1)%observed vector size OK
        x_m = x-repmat(pD(i).Mean,1,nx);
        W = pD(i).WishartW;
        v = pD(i).WishartDF;
        bi = pD(i).MeanPrec; % MeanPrec is only bi, and not bi*Lambdai
        
        % normalize W matrix to avoid overflow
        scale = max(max(W));
        Ws = W * (1/scale);
        K = size(W,1)*log(scale);
        %E1 = D*bi + v*diag((x_m)'*W*(x_m)); % Eq. 10.64
        E1 = D/bi + v*sum((x_m).*(W*(x_m))); % Eq. 10.64 (svantefix)
        E2 = D*log(2) + log(det(Ws))+ K + sum(psi((v+1-(1:D))/2)); % Eq. 10.65
        E2k(i)=E2;
        logP(i,:) = .5*E2 - .5*E1; % Part of Eq. 10.67
    else
        warning('Incompatible data size');
        logP(i,:)=repmat(-Inf,1,nx);%zero probability
    end;
end;