%function plotH=plotCross(pD,proj);
%2D plot of projected posterior Mean and Covariance
%for an array of trained GaussB objects
%Input:
%pD=        a single GaussB object or multidim array of such objects
%proj=      vector with indices of vector elements to plot
%           length(proj)==2 (but 3 should maybe also be allowed
%colors=    string with plot color codes, 'rgb' etc
%
%Result:
%plotH=    vector with plot handles to graph objects
%
%Arne Leijon 2007-05-25 slightly tested

function [plotH]=plotCross(pD,proj,colors)
if nargin<2
    proj=[1 2];%default coords
end;
if length(proj)~=2
    error('Can only plot 2-D');%should maybe allow 3 dims?
end;
if nargin<3
    colors='rbgk';
end;
if length(colors)==0
    colors='rbgk';
end;
nObj=numel(pD);
plotH=[];%storage for plot handles
for i=1:nObj
    beta=pD(i).MeanPrec;%training data weight in this component
    if beta>1 % otherwise not much to plot....
        cObj=colors(1+mod(i-1,length(colors)));%circulate among given colors
        m=pD(i).Mean(proj);%keep only 2 selected dimensions
        D=size(m,1);
        C=pD(i).WishartW*(pD(i).WishartDF+1-D)*beta./(1+beta);%posterior precision matrix
        C=C(proj,proj);%keep only 2 selected dimensions
        [v,d]=eig(C);%eigenvectors and eigenvalues
        v=v*diag(sqrt(1./diag(d)));%scale principal coordinate axes to indicate +-1 st.dev
        for k=1:size(v,2)%for each main axis
            xx=[(m-v(:,k)),(m+v(:,k))];%line start,end points
            xx=xx(proj,:);%selected coords for plot
            plotH=[plotH,plot(xx(1,:)',xx(2,:)',['-',cObj],'LineWidth',2)];            
            hold on;
        end;        
    end;    
end;
% comnum=length(plotH)/2;
hold off;
