%[f,logS]=prob(pD,x) gives probability densities for given vectors
%to be drawn from a given GaussD object.
%
%Input:
%pD=    GaussD object or multidim array of GaussD objects
%x=     matrix with given vectors stored columnwise
%
%Result:
%f=     proportional to probability densities for x
%       size(f)== [length(pD),size(x,2)], if pD is one-dimensional vector
%       size(f)== [size(pD),size(x,2)], if pD is multidim array
%       max value of f is always 1.
%logS=  scalar log scalefactor to make sure f is within numerical limits
%       f*exp(logS)=   true probability densities for x
%
%This representation is necessary because the probability density may be
%extremely small for random vectors with many elements
%  
%Arne Leijon 2009-05-06 tested

function [f,logS]=prob(pD,x)
logf=logprob(pD,x);
logS=max(logf(:));
if logS==-Inf
    f=ones(size(logf));%f*exp(logS)=0 as desired
else
    f=exp(logf-logS);%largest value always =1
end;
