%OK=propertiesOK(gD)
%tests if all properties are consistent 
%in given GaussB object or array of such objects.
%Input:
%gD=  GaussD object(s)
%
%Result:
%OK=    true, if all properties are OK
%warnings are signalled, if not OK
%
%Arne Leijon, 2007-05-23 tested

function OK=propertiesOK(gD)
nObj=numel(gD);
OK=true;
for i=1:nObj
    if isempty(gD(i).Mean)
        OK=false;
        warning('No Mean value');return;
    end;%cannot check further
    if isempty(gD(i).MeanPrec)
        OK=false;
        warning('No MeanPrec value');return;
    end;%cannot check further
    if isempty(gD(i).WishartW)
        OK=false;
        warning('No WishartW value');return;
    end;%cannot check further
    if isempty(gD(i).WishartDF)
        OK=false;
        warning('No WishartDF value');return;
    end;%cannot check further
    if length(gD(i).Mean) ~= size(gD(i).WishartW,1)
        OK=false;
        warning('Mean and WishartW must have compatible size');
    end;
    if size(gD(i).WishartW,1) ~= size(gD(i).WishartW,2)
        OK=false;
        warning('WishartW must be square');
    end;
    if numel(gD(i).MeanPrec) ~= 1
        OK=false;
        warning('MeanPrec must be scalar');
    end;
    if numel(gD(i).WishartDF) ~= 1
        OK=false;
        warning('WishartDF must be scalar');
    end;
end;