%VQ   Constructor for a single VQ object.
%
%A VQ object represents a Vector Quantizer, that can be used
%to encode each continuous-valued vector into a corresponding integer code index.
%
%***Usage:
%vq = VQ; 
%   creates a single empty vector quantizer.
%vq = VQ(vq); 
%   just copies the given object.
%vq = VQ(codebook)
%   creates a VQ object from given CodeBook matrix
%vq = VQ('CodeBook',codebook)
%   creates a VQ object with given class properties.
%
%***Class Properties:
%CodeBook=  matrix with cluster centroids, stored column-wise
%(***** distance measure may be added, if not Euclidean distance ****)
%
%***Input:
%vq=           (optional) VQ object or array of such objects
%propertyName, propertyValue = (optional) name string, corresp. value.
%
%Result:
%vq=            the new VQ object
%
%Internal representation:
%vq.CodeBook=   matrix with entries as column vectors
%
%Arne Leijon 2006-08-08 tested

function vq = VQ(varargin)
switch nargin
    case 0
        vq= class(initVQstruct,'VQ');%empty codebook
    case 1
        switch class(varargin{1});
            case {'VQ','vq'}
                vq=varargin{1};%just keep it, if it is a VQ
            case 'double'
                vq.CodeBook=varargin{1};%it is a Codebook
                vq=class(vq,'VQ');
            otherwise
                error('Single argument must be a VQ or a CodeBook matrix');
        end;%type switch
    otherwise
        vq=class(initVQstruct,'VQ');%create empty
        vq=set(vq,varargin{:});%and set properties
 end;%switch

function g=initVQstruct
g.CodeBook=[];%empty codebook
