%pD=set(pD,propName,value,...) sets value of property propName
%for a single VQ object or array of VQ objects
%
%Input:
%pD=    the object or object array
%propName, value=   property name, property value
%If value is a numeric array,
%       ALL elements in the object array pD are assigned the new property value.
%If value is a cell array, it must have the same size as the VQ array,
%       and then each object is assigned with the corresponding value cell.
%
%Several (propName,value) argument pairs may follow
%(******* although so far we have only the CodeBook property ******).
%
%Result:
%pD=    modified object(s)
%OK=    TRUE, if properties are acceptable in all pD elements
%
%Arne Leijon 2006-03-31 tested

function pD=set(pD,varargin);
sizObj=size(pD);%object array size
nObj=prod(sizObj);%number of objects in array
property_argin = varargin;
while length(property_argin) >= 2,
    propName = property_argin{1};
    v = property_argin{2};
    property_argin = property_argin(3:end);
    if iscell(v)%check size
        if any(sizObj~=size(v))
            error([propName,' cell array has incompatible size']);
        end;
    end;
    switch propName
        case 'CodeBook'
            if iscell(v)
                for i=1:nObj
                    pD(i).CodeBook=v{i};%each object gets separate value
                end;
            else
                for i=1:nObj
                    pD(i).CodeBook=v;%all objects get same value
                end;
            end;
        otherwise
            warning([propName,' is not a property of this object']);
    end;%switch
end;%while
