%function [vq,varC]=trainLloyd(vq,X);
%trains a Vector Quantizer for vectors in matrix X,
%for minimum Euclidean square-sum distortion.
%The vq must be previously suitably initialized,
%e.g. by a call to method init.
%
%Input:
%vq=    a VQ object, possibly empty
%X=     matrix with training vectors, stored column-wise
%
%Result:
%vq=    new VQ object, adapted to given training data.
%varC=  square deviations for each VQ cluster center
%       size(varC)==size(C.CodeBook)
%
%Normally, size of CodeBook is unchanged by training, BUT
%   CodeBook may be reduced during training, if a cluster is empty.
%
%Method: Generalized Lloyd algorithm is used
%
%Arne Leijon 2006-04-03 tested

function [vq,varC]=trainLloyd(vq,X)
if size(vq.CodeBook,1)~=size(X,1)
    error('Incompatible vector length');end;
initialCodeBookSize=size(vq.CodeBook,2);
if initialCodeBookSize> size(X,2)/2
    warning('Too few training vectors -- result may be inaccurate');end;

iX=encode(vq,X);%initial encoding
iXprevious=zeros(size(iX));%nonsense, just for start
while any(iX~=iXprevious)%try again, new step in Lloyd algorithm
	%iXwrong=sum(iX~=iXprevious%for testing
    wstate=warning;
	warning off;%because we check div by 0 later
	for n=1:size(vq.CodeBook,2)
		vq.CodeBook(:,n)=mean(X(:,iX==n),2);%modified codebook entry
%		vq.CodeBook(:,n)==NaN, if there were no training vectors
	end;
	vq.CodeBook=vq.CodeBook(:,isfinite(vq.CodeBook(1,:)));%remove NaN code vectors
	warning(wstate);%restore
	iXprevious=iX;%remember previous encoding
	iX=encode(vq,X);%encoding with new codebook
end;%no more improvement possible
if size(vq.CodeBook,2)<initialCodeBookSize
    warning('CodeBook has been reduced during training');end;
%
varC=zeros(size(vq.CodeBook));%mean square deviations from cluster centres
for n=1:size(vq.CodeBook,2)
    d=X(:,iX==n)-repmat(vq.CodeBook(:,n),1,sum(iX==n));
    varC(:,n)=mean(d.*d,2);
end;